library ieee;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.NUMERIC_STD.ALL; 
 
entity noteToneGenerator is 
	port (  
			clk			: in	STD_LOGIC;
			noteIn		: in	STD_LOGIC_VECTOR ( 2 downto 0 );
			amplitude	: in	STD_LOGIC_VECTOR ( 7 downto 0 );
			waveOut		: out	STD_LOGIC_VECTOR ( 7 downto 0 )
         ); 
end noteToneGenerator; 
 
architecture Behavioral of noteToneGenerator is 

constant NOTE_C4 : integer := 53200000/(2*262)-1 ;
constant NOTE_D4 : integer := 53200000/(2*294)-1 ;
constant NOTE_E4 : integer := 53200000/(2*330)-1 ;
constant NOTE_F4 : integer := 53200000/(2*349)-1 ;
constant NOTE_G4 : integer := 53200000/(2*392)-1 ;
constant NOTE_A4 : integer := 53200000/(2*440)-1 ;
constant NOTE_H4 : integer := 53200000/(2*494)-1 ;
constant NOTE_C5 : integer := 53200000/(2*523)-1 ;

constant	MINFREQUENCY	: integer := 10; -- Hz
signal		prescaler		: integer range 0 to 53200000/(2*MINFREQUENCY)-1 := 53200000/(2*MINFREQUENCY)-1 ; --  bei 53.2MHz fosc

signal		freqCounter	: integer range 0 to 53200000/(2*MINFREQUENCY)-1 := 0; 

constant	MAXNOTE		: integer := 7 ;
signal		note		: integer range 0 to MAXNOTE := 1 ;

signal		bitSignal	: bit := '0';

begin 

   soundGenerator:	process begin   
						wait until rising_edge( clk ); 
							if ( freqCounter < prescaler) then  
								freqCounter <= freqCounter+1;                
							else                         
								freqCounter <= 0;                  
								bitSignal <= not bitSignal;  -- toggle bitSignal
							end if; 
					end process soundGenerator ; 
   
   	waveOut <= "00000000" when bitSignal ='0' else amplitude ;

	note <= to_integer( unsigned(noteIn) ) ;
   
	-- note to frequency translation
	with note select 
		prescaler <=	NOTE_C4 when 0 ,
						NOTE_D4 when 1 ,
						NOTE_E4 when 2 ,
						NOTE_F4 when 3 ,
						NOTE_G4 when 4 ,
						NOTE_A4 when 5 ,
						NOTE_H4 when 6 ,
						NOTE_C5 when others ;     	
				  
end Behavioral;