library ieee;
use IEEE.STD_LOGIC_1164.ALL; 
use IEEE.NUMERIC_STD.ALL; 

-- LIBRARY lattice;
-- USE lattice.components.all;

entity melodyPlayer is
	Port	(  
				system_clk			: in STD_LOGIC;
				system_leds			: out STD_LOGIC_VECTOR ( 7 downto 0 );
				system_pwmSpeaker	: out STD_LOGIC
			); 
end entity melodyPlayer;

architecture Behaviour of melodyPlayer is
	
	component sequencer
		port	(  
					clk     : in	STD_LOGIC ;
					syncOut	: out	STD_LOGIC;
					leds    : out	STD_LOGIC_VECTOR ( 7 downto 0 )
				); 		
	end component;
		
	component noteToneGenerator  
		port (  
				clk			: in	STD_LOGIC;
				noteIn		: in	STD_LOGIC_VECTOR ( 2 downto 0 );
				amplitude	: in	STD_LOGIC_VECTOR ( 7 downto 0 );
				waveOut		: out	STD_LOGIC_VECTOR ( 7 downto 0 )
			 ); 
	end component;
   
	component pwmDac
		port	( 
					clk		: in STD_LOGIC;
					pwmValue: in STD_LOGIC_VECTOR ( 9 downto 0 );
					pwmOut	: out STD_LOGIC
				);
	end component;
	
	component adsr
		port(
			reset    : in  std_logic;
			clk      : in  std_logic;
			attack   : in  std_logic_vector(7 downto 0);
			decay    : in  std_logic_vector(7 downto 0);
			sustain  : in  std_logic_vector(7 downto 0);
			release  : in  std_logic_vector(7 downto 0);
			velocity : in  std_logic_vector(7 downto 0);
			input    : in  std_logic_vector(7 downto 0);
			output   : out std_logic_vector(7 downto 0)
		);
	end component;
   
	signal	sequencerStep	: STD_LOGIC_VECTOR ( 7 downto 0 );
	signal	system_pwmValue	: STD_LOGIC_VECTOR ( 9 downto 0 );
	signal	pwmValueInt		: integer range 0 to 1023;
	signal	note1			: STD_LOGIC_VECTOR ( 2 downto 0 ) := "011";
	signal	wave1			: STD_LOGIC_VECTOR ( 7 downto 0 );
	signal	amplitude1		: STD_LOGIC_VECTOR ( 7 downto 0 ) := "01000000";
	signal	note2			: STD_LOGIC_VECTOR ( 2 downto 0 ) := "011";
	signal	wave2			: STD_LOGIC_VECTOR ( 7 downto 0 );
	signal	amplitude2		: STD_LOGIC_VECTOR ( 7 downto 0 ) := "01000000";

	
	signal		stepSync	: STD_LOGIC :='0';

begin	


    adsr0: adsr
		port map (
					reset		=> stepSync,
					clk			=> system_clk,
					attack		=> "00001000", -- time
					decay		=> "10001000", -- time
					sustain		=> "00000001", -- level after attack/decay
					release		=> "00001000", -- time to zero level after key released
					velocity	=> "11111111", -- how fast you hit the key
					input		=> "11111111", -- level
					output		=>	amplitude1
				);
				
    adsr1: adsr
		port map (
					reset		=> stepSync,
					clk			=> system_clk,
					attack		=> "00001000", -- time
					decay		=> "11001000", -- time
					sustain		=> "00000001", -- level after attack/decay
					release		=> "01000000", -- time to zero level after key released
					velocity	=> "11111111", -- how fast you hit the key
					input		=> "11111111", -- level
					output		=>	amplitude2
				);


	sequencer0: sequencer
		port map	( clk => system_clk, syncOut => stepSync, leds => sequencerStep );
		
    noteToneGenerator0: noteToneGenerator
    	port map	( clk => system_clk, noteIn => note1, amplitude => amplitude1, waveOut => wave1 );

    noteToneGenerator1: noteToneGenerator
    	port map	( clk => system_clk, noteIn => note2, amplitude => amplitude2, waveOut => wave2 );
		
	pwmDac0: pwmDac
		port map	( clk => system_clk, pwmValue => system_pwmValue, pwmOut => system_pwmSpeaker );
	
	-- show sequencer steps
	system_leds <= sequencerStep;
	
	-- mixer
	pwmValueInt <= to_integer(unsigned(wave1))+ to_integer(unsigned(wave2));
	system_pwmValue (7 downto 0 ) <= std_logic_vector( to_unsigned( pwmValueInt,8 ) );
	
	-- use sequencer leds to control sound
	-- melody1
	 with sequencerStep select 
		note1 <=	"000" when not "00000001" ,
					"000" when not "00000010" , 
					"000" when not "00000100" ,
					"001" when not "00001000" ,
					
					"011" when not "00010000" ,
					"000" when not "00100000" ,
					--"010" when not "0100_0000" ,
					--"011" when not "1000_0000" ,
					"000" when others ; 
     -- melody2
	 with sequencerStep select 
		note2 <=	"000" when not "00000001" ,
					"010" when not "00000010" ,
					"101" when not "00000100" ,
					"011" when not "00001000" ,
					
					"101" when not "00010000" ,
					"010" when not "00100000" ,
					--"010" when not "01000000" ,
					--"011" when not "10000000" ,
					"000" when others ;
					
end architecture;
